
##
## compute inducible pairs from compatible pairs as a stabilizer with
## finite index
##
InduciblePairsCG := function(G, com)
    local S, C, c, tup, cf, act, l;

    # turn com into group
    C := Group(com);
    SetSize(C, Size(NormalizerOfPointGroup(G)));

    # add info
    c := TwoCohomInfoCG(G);
    if c = 0*c then
        #Print(" inducible pairs index ",1," \n");
        return C;
    fi;

    # get tuples
    tup := List(com, x -> MatrixOperOfTupleCG(G,x));
    if ForAll( tup, x -> x = x^0 ) then 
        #Print(" inducible pairs index ",1," \n");
        return C; 
    fi;

    # determine stabilizer
    cf := G!.cohom;
    act := function( vec, mat )
        local new, i;
        new := ShallowCopy( vec * mat );
        for i in [1..Length(new)] do
            new[i] := new[i] mod cf.rels[i];
        od;
        return new;
    end;

    l := Length(Orbit(C, c, com, tup, act));
    #Print(" inducible pairs index ",l," \n");

    return Stabilizer(C, c, com, tup, act);
end;

##
## technical detail ...
##
RelatorMatrix := function(C)
    local n, d, l, m, f, E, k, e, i, j, h, A, B, r, w, s, t;

    if IsBound(C!.relma) then return C!.relma; fi;

    n := Length(C.f);
    d := Length(C.t);
    l := Length(C.enumrels);
    m := C.mats;
    f := RelativeOrdersOfPcp(C.factor);
    E := [];

    for k in [1..l] do
        A := List([1..n], x -> NullMat(d,d));
        e := C.enumrels[k];
        i := e[1]; j := e[2];
        if j < i then
            A[j] := IdentityMat(d) - m[i]^m[j];
            A[i] := m[j];
        elif j > i then
            A[j-i] := m[i]*m[j-i]^-1 - m[j-i]^-1;
            A[i] := m[j-i]^-1;
        else
            for h in [0..f[i]-1] do
                A[i] := A[i] + m[i]^h;
            od;
        fi;

        B := List([1..n], x -> NullMat(d,d));
        r := C.relators[i][j];
        w := IdentityMat(d);
        for s in Reversed([1..Length(r)]) do
            for t in [1..r[s][2]] do
                B[r[s][1]] := B[r[s][1]] + w;
                w := m[r[s][1]] * w;
            od;
        od;

        E[k] := A-B;
    od;
    C!.relma := E;
    return E;
end;


##
## lift inducible pair to an automorphism
##
LiftAutoCG := function( G, tup )
    local C, n, d, l, i, a, b, v, E, F, k, j, h, c, s, L;

    # get info
    AddExtensionInfoCG(G);
    C := G!.crrec;
    n := Length(C.f);
    d := Length(C.t);
    l := Length(C.enumrels);
    k := [];

    # set up
    for i in [1..n] do
        k[i] := MappedVector(Exponents(Image(tup[2], C.factor[i])), C.f);
    od;
    for i in [1..d] do
        k[n+i] := MappedVector(tup[1][i], C.t);
    od;

    # the trivial case
    if n = 0 then return
        GroupHomomorphismByImagesNC( G, G, C.h, k );
    fi;

    # tail
    a := List( C.enumrels, x -> TailOfRelator( C, k, x[1], x[2] ) );
    b := List( C.enumrels, x -> TailOfRelator( C, C.h, x[1], x[2] )*tup[1] );
    v := Concatenation(b-a);


    # relator mat
    E := StructuralCopy(RelatorMatrix(C));
    F := NullMat( n*d, l*d );
    for s in [1..l] do
        for i in [1..n] do
            L := E[s][i]^tup[1];
            for j in [1..d] do
                for h in [1..d] do
                    F[d*(i-1)+j][d*(s-1)+h] := L[j][h];
                od;
            od;
        od;
    od;

    # solve
    c := SolutionIntMat( F, v );
    if IsBool(c) then Error("no solution"); fi;
    c := CutVector(c, n);

    # add correction
    for i in [1..n] do
        k[i] := k[i]*MappedVector(c[i], C.t);
    od;

    # construct homomorphism
    a := GroupGeneralMappingByImagesNC(G,G,C.h,k);
    #a := GroupHomomorphismByImagesNC( G, G, C.h, k);
    if not IsGroupHomomorphism(a) or not IsBijective(a) then
        Error("hier");
    fi;
    return a;
end;


